@@ -21,6 +21,10 @@ from oslo_utils import strutils
 
 from proliantutils import exception
 from proliantutils.hpssa import constants
+from proliantutils import log
+
+
+LOG = log.get_logger(__name__)
 
 
 def _get_indentation(string):
@@ -37,7 +41,7 @@ def _get_key_value(string):
     key = ''
     value = ''
     try:
-        key, value = string.split(':')
+        key, value = string.split(': ')
     except ValueError:
         # This handles the case when the property of a logical drive
         # returned is as follows. Here we cannot split by ':' because
@@ -47,16 +51,18 @@ def _get_key_value(string):
         string = string.lstrip(' ')
         if string.startswith('physicaldrive'):
             fields = string.split(' ')
-            key = fields[0]
+            # Include fields[1] to key to avoid duplicate pairs
+            # with the same 'physicaldrive' key
+            key = fields[0] + " " + fields[1]
             value = fields[1]
         else:
             # TODO(rameshg87): Check if this ever occurs.
-            return None, None
+            return string.strip(' '), None
 
-    return key.lstrip(' ').rstrip(' '), value.lstrip(' ').rstrip(' ')
+    return key.strip(' '), value.strip(' ')
 
 
-def _get_dict(lines, start_index, indentation):
+def _get_dict(lines, start_index, indentation, deep):
     """Recursive function for parsing hpssacli/ssacli output."""
 
     info = {}
@@ -68,37 +74,38 @@ def _get_dict(lines, start_index, indentation):
         current_line = lines[i]
         current_line_indentation = _get_indentation(current_line)
 
+        # Check for multi-level returns
+        if current_line_indentation < indentation:
+            return info, i-1
+
         if current_line_indentation == indentation:
             current_item = current_line.lstrip(' ')
             info[current_item] = {}
             i = i + 1
             continue
 
-        if i >= len(lines) - 1:
+        if i < len(lines) - 1:
+            next_line_indentation = _get_indentation(lines[i+1])
+        else:
+            next_line_indentation = current_line_indentation
+
+        if next_line_indentation > current_line_indentation:
+            ret_dict, i = _get_dict(lines, i, current_line_indentation, deep+1)
+            for key in ret_dict.keys():
+                if key in info[current_item]:
+                    info[current_item][key].update(ret_dict[key])
+                else:
+                    info[current_item][key] = ret_dict[key]
+        else:
             key, value = _get_key_value(current_line)
-            # If this is some unparsable information, then
-            # just skip it.
             if key:
                 info[current_item][key] = value
+
+        # Do not return if it's the top level of recursion
+        if next_line_indentation < current_line_indentation and deep > 0:
             return info, i
 
-        next_line = lines[i+1]
-        next_line_indentation = _get_indentation(next_line)
-
-        if current_line_indentation == next_line_indentation:
-            key, value = _get_key_value(current_line)
-            if key:
-                info[current_item][key] = value
-            i = i + 1
-        elif next_line_indentation > current_line_indentation:
-            ret_dict, j = _get_dict(lines, i, current_line_indentation)
-            info[current_item].update(ret_dict)
-            i = j + 1
-        elif next_line_indentation < current_line_indentation:
-            key, value = _get_key_value(current_line)
-            if key:
-                info[current_item][key] = value
-            return info, i
+        i = i + 1
 
     return info, i
 
@@ -113,7 +120,7 @@ def _convert_to_dict(stdout):
 
     lines = stdout.split("\n")
     lines = list(filter(None, lines))
-    info_dict, j = _get_dict(lines, 0, 0)
+    info_dict, j = _get_dict(lines, 0, 0, 0)
     return info_dict
 
 
@@ -444,26 +451,38 @@ class Controller(object):
     def erase_devices(self, drives):
         """Perform Erase on all the drives in the controller.
 
-        This method erases all the drives in the controller by overwriting
-        the drives with the pattern. The drives will be unavailable until
-        after successful completion or failure. The possible erase pattern
-        available for sanitize erase are 'overwrite' and 'block' to perform
-        erase on HDD and SSD respectively.
+        This method erases all the hdd and ssd drives in the controller
+        by overwriting the drives with patterns for hdd and erasing storage
+        blocks for ssd drives. The drives would be unavailable until
+        successful completion or failure of erase operation.
 
-        If the sanitize erase is not supported on the controller it performs
-        the disk erase by overwritting with zeros.
+        If the sanitize erase is not supported on any disk it will try to
+        populate zeros on disk drives.
 
         :param drives: A list of drive objects in the controller.
+        :raises: HPSSAOperationError, if sanitize erase is not supported.
         """
-        # TODO(aparnav): Add Sanitize erase support for SSD
-        drive = ','.join([x.id for x in drives])
-        cmd_args = self._get_erase_command(drive, 'erasepattern=overwrite')
+        for drive in drives:
+            pattern = 'overwrite' if (
+                drive.disk_type == constants.DISK_TYPE_HDD) else 'block'
+            cmd_args = self._get_erase_command(
+                drive.id, 'erasepattern=%s' % pattern)
+            stdout = self.execute_cmd(*cmd_args)
+            LOG.debug("Sanitize disk erase invoked with erase pattern as "
+                      "'%(pattern)s' on disk type: %(disk_type)s."
+                      % {'pattern': pattern, 'disk_type': drive.disk_type})
 
-        stdout = self.execute_cmd(*cmd_args)
-        if "not supported" in str(stdout):
-            cmd_args = self._get_erase_command(drive,
-                                               'erasepattern=zero')
-            self.execute_cmd(*cmd_args)
+            if "not supported" in str(stdout):
+                new_pattern = 'zero'
+                cmd_args = self._get_erase_command(drive.id,
+                                                   'erasepattern=zero')
+                self.execute_cmd(*cmd_args)
+                LOG.debug("Sanitize disk erase invoked with erase pattern as "
+                          "'%(pattern)s' is not supported on disk type: "
+                          "%(disk_type)s. Now its invoked with erase pattern "
+                          "as %(new_pattern)s."
+                          % {'pattern': pattern, 'disk_type': drive.disk_type,
+                             'new_pattern': new_pattern})
 
 
 class RaidArray(object):
@@ -556,14 +575,22 @@ class LogicalDrive(object):
         # 'string_to_bytes' takes care of converting any returned
         # (like 500MB, 25GB) unit of storage space to bytes (Integer value).
         # It requires space to be stripped.
-        size = self.properties['Size'].replace(' ', '')
         try:
+            size = self.properties['Size'].replace(' ', '')
             # TODO(rameshg87): Reduce the disk size by 1 to make sure Ironic
             # has enough space to write a config drive. Remove this when
             # Ironic doesn't need it.
             self.size_gb = int(strutils.string_to_bytes(size,
                                                         return_int=True) /
                                (1024*1024*1024)) - 1
+        except KeyError:
+            msg = ("Can't get 'Size' parameter from ssacli output for logical "
+                   "disk '%(logical_disk)s' of RAID array '%(array)s' in "
+                   "controller '%(controller)s'." %
+                   {'logical_disk': self.id,
+                    'array': self.parent.id,
+                    'controller': self.parent.parent.id})
+            raise exception.HPSSAOperationError(reason=msg)
         except ValueError:
             msg = ("ssacli returned unknown size '%(size)s' for logical "
                    "disk '%(logical_disk)s' of RAID array '%(array)s' in "
@@ -617,14 +644,21 @@ class PhysicalDrive(object):
         # Strip off physicaldrive before storing it in id
         self.id = id[14:]
 
-        size = self.properties['Size'].replace(' ', '')
         # 'string_to_bytes' takes care of converting any returned
         # (like 500MB, 25GB) unit of storage space to bytes (Integer value).
         # It requires space to be stripped.
         try:
+            size = self.properties['Size'].replace(' ', '')
             self.size_gb = int(strutils.string_to_bytes(size,
                                                         return_int=True) /
                                (1024*1024*1024))
+        except KeyError:
+            msg = ("Can't get 'Size' parameter from ssacli output for "
+                   "physical disk '%(physical_disk)s' of controller "
+                   "'%(controller)s'." %
+                   {'physical_disk': self.id,
+                    'controller': self.parent.parent.id})
+            raise exception.HPSSAOperationError(reason=msg)
         except ValueError:
             msg = ("ssacli returned unknown size '%(size)s' for physical "
                    "disk '%(physical_disk)s' of controller "
@@ -633,7 +667,16 @@ class PhysicalDrive(object):
                     'controller': self.parent.id})
             raise exception.HPSSAOperationError(reason=msg)
 
-        ssa_interface = self.properties['Interface Type']
+        try:
+            ssa_interface = self.properties['Interface Type']
+        except KeyError:
+            msg = ("Can't get 'Interface Type' parameter from ssacli output "
+                   "for physical disk '%(physical_disk)s' of controller "
+                   "'%(controller)s'." %
+                   {'physical_disk': self.id,
+                    'controller': self.parent.parent.id})
+            raise exception.HPSSAOperationError(reason=msg)
+
         self.interface_type = constants.get_interface_type(ssa_interface)
         self.disk_type = constants.get_disk_type(ssa_interface)
         self.model = self.properties.get('Model')
