@@ -13,6 +13,8 @@
 # under the License.
 """IloClient module"""
 
+import collections
+import netaddr
 from proliantutils import exception
 from proliantutils.ilo import ipmi
 from proliantutils.ilo import operations
@@ -25,8 +27,10 @@ from proliantutils.redfish import redfish
 SUPPORTED_RIS_METHODS = [
     'activate_license',
     'clear_secure_boot_keys',
+    'create_raid_configuration',
     'delete_raid_configuration',
     'eject_virtual_media',
+    'get_bios_settings_result',
     'get_current_bios_settings',
     'get_current_boot_mode',
     'get_host_post_state',
@@ -47,6 +51,7 @@ SUPPORTED_RIS_METHODS = [
     'inject_nmi',
     'insert_virtual_media',
     'press_pwr_btn',
+    'read_raid_configuration',
     'reset_bios_to_default',
     'reset_ilo_credential',
     'reset_secure_boot_keys',
@@ -57,9 +62,7 @@ SUPPORTED_RIS_METHODS = [
     'set_one_time_boot',
     'set_pending_boot_mode',
     'set_secure_boot_mode',
-    'set_iscsi_boot_info',
     'set_iscsi_info',
-    'unset_iscsi_boot_info',
     'unset_iscsi_info',
     'get_iscsi_initiator_info',
     'set_iscsi_initiator_info',
@@ -69,7 +72,10 @@ SUPPORTED_RIS_METHODS = [
     ]
 
 SUPPORTED_REDFISH_METHODS = [
+    'create_raid_configuration',
     'delete_raid_configuration',
+    'do_disk_erase',
+    'has_disk_erase_completed',
     'get_product_name',
     'get_host_post_state',
     'get_host_power_status',
@@ -77,7 +83,9 @@ SUPPORTED_REDFISH_METHODS = [
     'reset_server',
     'press_pwr_btn',
     'hold_pwr_btn',
+    'get_bios_settings_result',
     'get_current_bios_settings',
+    'get_available_disk_types',
     'get_default_bios_settings',
     'get_pending_bios_settings',
     'set_bios_settings',
@@ -94,6 +102,7 @@ SUPPORTED_REDFISH_METHODS = [
     'set_one_time_boot',
     'update_persistent_boot',
     'set_pending_boot_mode',
+    'read_raid_configuration',
     'reset_ilo_credential',
     'reset_bios_to_default',
     'get_secure_boot_mode',
@@ -103,25 +112,92 @@ SUPPORTED_REDFISH_METHODS = [
     'get_server_capabilities',
     'get_supported_boot_mode',
     'get_essential_properties',
-    'set_iscsi_boot_info',
     'set_iscsi_info',
-    'unset_iscsi_boot_info',
     'unset_iscsi_info',
     'get_iscsi_initiator_info',
-    'set_iscsi_initiator_info',
+    'set_iscsi_initiator_info'
 ]
 
 LOG = log.get_logger(__name__)
 
 
+def cache_node(cache=True):
+
+    def wrapper(cls):
+
+        if not cache:
+            return cls
+        else:
+            class IloClientWrapper(object):
+
+                MAX_CACHE_SIZE = 1024
+
+                def __init__(self, cls):
+                    self.cls = cls
+                    self._instances = collections.OrderedDict()
+
+                def _if_not_exists(self, ilo_info):
+                    return (ilo_info not in self._instances)
+
+                def _create_instance(self, *args, **kwargs):
+                    address = args[0]
+                    admin = args[1]
+                    admin_pass = args[2]
+                    self._instances[(address, admin, admin_pass)] = (
+                        self.cls(*args, **kwargs))
+                    # Check for max_cache_size
+                    if len(self._instances) > self.MAX_CACHE_SIZE:
+                        LOG.debug("Node cache hit the maximum size of %d." % (
+                            self.MAX_CACHE_SIZE))
+                        self._pop_oldest_node()
+
+                def __call__(self, *args, **kwargs):
+                    if not args:
+                        LOG.error("Error creating iLO object.")
+                    address = args[0]
+                    admin = args[1]
+                    admin_pass = args[2]
+
+                    if self._if_not_exists((address, admin, admin_pass)):
+                        LOG.debug("Creating iLO object for node %(address)s.",
+                                  {'address': address})
+                        self._create_instance(*args, **kwargs)
+                    else:
+                        LOG.debug("Using existing object for node "
+                                  "%(address)s.", {'address': address})
+                    return self._instances[(address, admin, admin_pass)]
+
+                def _pop_oldest_node(self):
+                    node_keys = list(self._instances)
+                    node_key = next(iter(node_keys))
+                    LOG.debug("Removed oldest node {} from "
+                              "cache".format(node_key))
+                    rnode = self._instances.pop(node_key, None)
+                    if rnode:
+                        del rnode
+
+            return IloClientWrapper(cls)
+    return wrapper
+
+
+@cache_node()
 class IloClient(operations.IloOperations):
 
     def __init__(self, host, login, password, timeout=60, port=443,
                  bios_password=None, cacert=None, snmp_credentials=None,
                  use_redfish_only=False):
+
+        # IPv6 Check
+        # TODO(paresh) Need to test with Global IPv6 address
+        # IPMI supports IPv6 without square brackets
+        self.ipmi_host_info = {'address': host, 'username': login,
+                               'password': password}
+
+        if netaddr.valid_ipv6(host.split('%')[0]):
+            host = '[' + host + ']'
+
         self.ribcl = ribcl.RIBCLOperations(host, login, password, timeout,
                                            port, cacert=cacert)
-        self.info = {'address': host, 'username': login, 'password': password}
         self.host = host
         self.use_redfish_only = use_redfish_only
 
@@ -163,6 +239,13 @@ class IloClient(operations.IloOperations):
         LOG.debug(self._("IloClient object created. "
                          "Model: %(model)s"), {'model': self.model})
 
+    def __del__(self):
+        try:
+            if self.redfish:
+                del self.redfish
+        except AttributeError:
+            pass
+
     def _init_redfish_object(self, is_ribcl_enabled, redfish_controller_ip,
                              username, password, bios_password=None,
                              cacert=None, should_set_model=True):
@@ -291,7 +374,7 @@ class IloClient(operations.IloOperations):
 
     def set_iscsi_info(self, target_name, lun, ip_address,
                        port='3260', auth_method=None, username=None,
-                       password=None):
+                       password=None, macs=[]):
         """Set iscsi details of the system in uefi boot mode.
 
         The initiator system is set with the target details like
@@ -303,60 +386,24 @@ class IloClient(operations.IloOperations):
         :param auth_method : either None or CHAP.
         :param username: CHAP Username for authentication.
         :param password: CHAP secret.
+        :param mac: List of target mac for iSCSI.
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedInBiosError, if the system is
                  in the bios boot mode.
         """
         return self._call_method('set_iscsi_info', target_name, lun,
                                  ip_address, port, auth_method, username,
-                                 password)
+                                 password, macs)
 
-    def set_iscsi_boot_info(self, mac, target_name, lun, ip_address,
-                            port='3260', auth_method=None, username=None,
-                            password=None):
-        """Set iscsi details of the system in uefi boot mode.
-
-        The initiator system is set with the target details like
-        IQN, LUN, IP, Port etc.
-        :param mac: The MAC of the NIC to be set with iSCSI information
-        :param target_name: Target Name for iscsi.
-        :param lun: logical unit number.
-        :param ip_address: IP address of the target.
-        :param port: port of the target.
-        :param auth_method : either None or CHAP.
-        :param username: CHAP Username for authentication.
-        :param password: CHAP secret.
-        :raises: IloError, on an error from iLO.
-        :raises: IloCommandNotSupportedInBiosError, if the system is
-                 in the bios boot mode.
-        """
-        LOG.warning("'set_iscsi_boot_info' is deprecated. The 'MAC' parameter"
-                    "passed in is ignored. Use 'set_iscsi_info' instead.")
-        return self._call_method('set_iscsi_info', target_name, lun,
-                                 ip_address, port, auth_method, username,
-                                 password)
-
-    def unset_iscsi_info(self):
+    def unset_iscsi_info(self, macs=[]):
         """Disable iscsi boot option of the system in uefi boot mode.
 
+        :param mac: List of target mac for iSCSI.
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedInBiosError, if the system is
                  in the bios boot mode.
         """
-        return self._call_method('unset_iscsi_info')
-
-    def unset_iscsi_boot_info(self, mac):
-        """Disable iscsi boot option of the system in uefi boot mode.
-
-        :param mac: The MAC of the NIC to be set with iSCSI information.
-        :raises: IloError, on an error from iLO.
-        :raises: IloCommandNotSupportedInBiosError, if the system is
-                 in the bios boot mode.
-        """
-        LOG.warning("'unset_iscsi_boot_info' is deprecated. The 'MAC' "
-                    "parameter passed in is ignored. Use 'unset_iscsi_info' "
-                    "instead.")
-        return self._call_method('unset_iscsi_info')
+        return self._call_method('unset_iscsi_info', macs)
 
     def get_iscsi_initiator_info(self):
         """Returns iSCSI initiator information of iLO.
@@ -626,7 +673,8 @@ class IloClient(operations.IloOperations):
             # NOTE(vmud213): Even if it is None, pass it on to get_nic_capacity
             # as we still want to try getting nic capacity through ipmitool
             # irrespective of what firmware we are using.
-            nic_capacity = ipmi.get_nic_capacity(self.info, major_minor)
+            nic_capacity = ipmi.get_nic_capacity(self.ipmi_host_info,
+                                                 major_minor)
             if nic_capacity:
                 capabilities.update({'nic_capacity': nic_capacity})
 
@@ -652,6 +700,39 @@ class IloClient(operations.IloOperations):
         """
         return self._call_method('delete_raid_configuration')
 
+    def create_raid_configuration(self, raid_config):
+        """Create the raid configuration on the hardware.
+
+        :param raid_config: A dictionary containing target raid configuration
+                            data. This data stucture should be as follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the command is not supported
+                 on the server
+        """
+        return self._call_method('create_raid_configuration', raid_config)
+
+    def read_raid_configuration(self, raid_config=None):
+        """Read the logical drives from the system.
+
+        :param raid_config: None in case of post-delete read or in case of
+                            post-create a dictionary containing target raid
+                            configuration data. This data stucture should be as
+                            follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the command is not supported
+                 on the server.
+        :returns: A dictionary containing list of logical disks
+        """
+        return self._call_method('read_raid_configuration', raid_config)
+
     def update_firmware(self, firmware_url, component_type):
         """Updates the given firmware on the server
 
@@ -688,7 +769,7 @@ class IloClient(operations.IloOperations):
         """
         return self._call_method('get_host_post_state')
 
-    def get_current_bios_settings(self, only_allowed_settings=True):
+    def get_current_bios_settings(self, only_allowed_settings=False):
         """Get current BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -705,7 +786,7 @@ class IloClient(operations.IloOperations):
         return self._call_method('get_current_bios_settings',
                                  only_allowed_settings)
 
-    def get_pending_bios_settings(self, only_allowed_settings=True):
+    def get_pending_bios_settings(self, only_allowed_settings=False):
         """Get current BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -722,7 +803,7 @@ class IloClient(operations.IloOperations):
         return self._call_method('get_pending_bios_settings',
                                  only_allowed_settings)
 
-    def set_bios_settings(self, data=None, only_allowed_settings=True):
+    def set_bios_settings(self, data=None, only_allowed_settings=False):
         """Sets current BIOS settings to the provided data.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -739,7 +820,7 @@ class IloClient(operations.IloOperations):
         return self._call_method('set_bios_settings', data,
                                  only_allowed_settings)
 
-    def get_default_bios_settings(self, only_allowed_settings=True):
+    def get_default_bios_settings(self, only_allowed_settings=False):
         """Get default BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -755,3 +836,40 @@ class IloClient(operations.IloOperations):
         """
         return self._call_method('get_default_bios_settings',
                                  only_allowed_settings)
+
+    def get_bios_settings_result(self):
+        """Gets the result of the bios settings applied
+
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the command is
+                 not supported on the server.
+        """
+        return self._call_method('get_bios_settings_result')
+
+    def has_disk_erase_completed(self):
+        """Get out-of-band sanitize disk erase status.
+
+        :returns: True if disk erase completed on all controllers
+                  otherwise False
+        :raises: IloError, on an error from iLO.
+        """
+        return self._call_method('has_disk_erase_completed')
+
+    def do_disk_erase(self, disk_type, pattern=None):
+        """Perform the out-of-band sanitize disk erase on the hardware.
+
+        :param disk_type: Media type of disk drives.
+        :param pattern: Erase pattern, if nothing passed default
+                        ('overwrite' for 'HDD', and 'block' for 'SSD') will
+                        be used.
+        :raises: IloError, on an error from iLO.
+        """
+        return self._call_method('do_disk_erase', disk_type, pattern)
+
+    def get_available_disk_types(self):
+        """Get the list of all disk type available in server
+
+        :returns: A list containing disk types.
+        :raises: IloError, on an error from iLO.
+        """
+        return self._call_method('get_available_disk_types')
