@@ -1209,17 +1209,58 @@ class RIBCLOperations(operations.IloOperations):
         """
         self._raise_command_not_supported("get_host_post_state")
 
+    def read_raid_configuration(self, raid_config=None):
+        """Read the logical drives from the system
+
+        Read raid configuration of the hardware.
+
+        :param raid_config: None in case of post-delete read or in case of
+                            post-create a dictionary containing target raid
+                            configuration data. This data stucture should be as
+                            follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloCommandNotSupportedError, if the command is not supported
+                 on the server.
+        """
+        self._raise_command_not_supported("read_raid_configuration")
+
     def delete_raid_configuration(self):
         """Delete the raid configuration on the hardware.
 
         Loops through each SmartStorageConfig controller and clears the
         raid configuration.
 
-        :raises: IloError, on an error from iLO
         :raises: IloCommandNotSupportedError
         """
         self._raise_command_not_supported("delete_raid_configuration")
 
+    def create_raid_configuration(self, raid_config):
+        """Create the raid configuration on the hardware.
+
+        Based on user raid_config input, it will create raid
+
+        :param raid_config: A dictionary containing target raid configuration
+                            data. This data stucture should be as follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloCommandNotSupportedError
+        """
+        self._raise_command_not_supported("create_raid_configuration")
+
+    def get_bios_settings_result(self):
+        """Gets the result of the bios settings applied
+
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the command is
+                 not supported on the server.
+        """
+        self._raise_command_not_supported("get_bios_settings_result")
+
 
 # The below block of code is there only for backward-compatibility
 # reasons (before commit 47608b6 for ris-support).
