@@ -312,6 +312,9 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
         :returns the tuple of SmartStorage URI, Headers and settings.
         """
         headers, storage_uri, storage_settings = self._get_storage_resource()
+        # Do not raise exception if there is no ArrayControllers
+        # as Storage can be zero at any point and if we raise
+        # exception it might fail get_server_capabilities().
         if ('links' in storage_settings and
                 'ArrayControllers' in storage_settings['links']):
             # Get the ArrayCOntrollers URI and Settings
@@ -323,28 +326,21 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                 raise exception.IloError(msg)
 
             return headers, array_uri, array_settings
-        else:
-            msg = ('"links/ArrayControllers" section in SmartStorage'
-                   ' does not exist')
-            raise exception.IloCommandNotSupportedError(msg)
 
     def _create_list_of_array_controllers(self):
         """Creates the list of Array Controller URIs.
 
-        :raises: IloCommandNotSupportedError if the ArrayControllers
-            doesnt have member "Member".
         :returns list of ArrayControllers.
         """
         headers, array_uri, array_settings = (
             self._get_array_controller_resource())
         array_uri_links = []
+        # Do not raise exception if there is no ArrayControllers
+        # as Storage can be zero at any point and if we raise
+        # exception it might fail get_server_capabilities().
         if ('links' in array_settings and
                 'Member' in array_settings['links']):
             array_uri_links = array_settings['links']['Member']
-        else:
-            msg = ('"links/Member" section in ArrayControllers'
-                   ' does not exist')
-            raise exception.IloCommandNotSupportedError(msg)
         return array_uri_links
 
     def _get_drive_type_and_speed(self):
@@ -396,6 +392,9 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
         """
         disk_details_list = []
         array_uri_links = self._create_list_of_array_controllers()
+        # Do not raise exception if there is no disk/logical drive
+        # as Storage can be zero at any point and if we raise
+        # exception it might fail get_server_capabilities().
         for array_link in array_uri_links:
             _, _, member_settings = (
                 self._rest_get(array_link['href']))
@@ -412,15 +411,6 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                         _, _, disk_details = (
                             self._rest_get(diskdrive_uri))
                         disk_details_list.append(disk_details)
-                else:
-                    msg = ('"links/Member" section in %s'
-                           ' does not exist', drive_name)
-                    raise exception.IloCommandNotSupportedError(msg)
-            else:
-                msg = ('"links/%s" section in '
-                       ' ArrayController/links/Member does not exist',
-                       drive_name)
-                raise exception.IloCommandNotSupportedError(msg)
         if disk_details_list:
             return disk_details_list
 
@@ -614,40 +604,106 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                    ' does not exist')
             raise exception.IloCommandNotSupportedError(msg)
 
-    def _change_iscsi_settings(self, iscsi_info):
+    def _get_uefi_device_path_by_mac(self, mac):
+        """Return uefi device path of mac.
+
+        :param mac: Mac address.
+        :returns: Uefi Device path.
+                  Ex. 'PciRoot(0x0)/Pci(0x2,0x3)/Pci(0x0,0x0)'
+        """
+        adapter_uri = '/rest/v1/Systems/1/NetworkAdapters'
+        for status, headers, member, memberuri in (
+                self._get_collection(adapter_uri)):
+            if status < 300 and member.get('PhysicalPorts'):
+                for port in member.get('PhysicalPorts'):
+                    if port['MacAddress'].lower() == mac.lower():
+                        return port['UEFIDevicePath']
+
+    def _get_nic_association_name_by_mac(self, mac):
+        """Return nic association name by mac address
+
+        :param mac: Mac address.
+        :returns: Nic association name. Ex. NicBoot1
+        """
+        headers, bios_uri, bios_settings = self._check_bios_resource()
+        mappings = (
+            self._get_bios_mappings_resource(
+                bios_settings).get('BiosPciSettingsMappings'))
+        correlatable_id = self._get_uefi_device_path_by_mac(mac)
+        for mapping in mappings:
+            for subinstance in mapping['Subinstances']:
+                for association in subinstance['Associations']:
+                    if subinstance.get('CorrelatableID') == correlatable_id:
+                        return [name for name in subinstance[
+                            'Associations'] if 'NicBoot' in name][0]
+
+    def _get_all_macs(self):
+        """Return list of macs available on system
+
+        :returns: List of macs
+        """
+        macs = []
+        adapter_uri = '/rest/v1/Systems/1/NetworkAdapters'
+        for status, headers, member, memberuri in (
+                self._get_collection(adapter_uri)):
+            if status < 300 and member.get('PhysicalPorts'):
+                for port in member.get('PhysicalPorts'):
+                    macs.append(port['MacAddress'].lower())
+        return macs
+
+    def _validate_macs(self, macs):
+        """Validate given macs are there in system
+
+        :param macs: List of macs
+        :raises: InvalidInputError, if macs not valid
+        """
+        macs_available = self._get_all_macs()
+        if not set(macs).issubset(macs_available):
+            msg = ("Given macs: %(macs)s not found in the system"
+                   % {'macs': list(set(macs) - set(macs_available))})
+            raise exception.InvalidInputError(msg)
+
+    def _change_iscsi_settings(self, iscsi_info, macs=[]):
         """Change iSCSI settings.
 
+        :param macs: List of target macs for iSCSI.
         :param iscsi_info: A dictionary that contains information of iSCSI
                            target like target_name, lun, ip_address, port etc.
         :raises: IloError, on an error from iLO.
         """
-        headers, bios_uri, bios_settings = self._check_bios_resource()
-        # Get the Mappings resource.
-        map_settings = self._get_bios_mappings_resource(bios_settings)
-        nics = []
-        for mapping in map_settings['BiosPciSettingsMappings']:
-            for subinstance in mapping['Subinstances']:
-                for association in subinstance['Associations']:
-                    if 'NicBoot' in association:
-                        nics.append(association)
+        iscsi_uri = self._check_iscsi_rest_patch_allowed()
+        association_names = []
+        if macs:
+            self._validate_macs(macs)
+            association_names = [
+                self._get_nic_association_name_by_mac(mac) for mac in macs]
+        else:
+            # Get the Mappings resource.
+            headers, bios_uri, bios_settings = self._check_bios_resource()
+            map_settings = self._get_bios_mappings_resource(bios_settings)
+            for mapping in map_settings['BiosPciSettingsMappings']:
+                for subinstance in mapping['Subinstances']:
+                    for association in subinstance['Associations']:
+                        if 'NicBoot' in association:
+                            association_names.append(association)
 
-        if not nics:
-            msg = ('No nics found')
+        if not association_names:
+            msg = ('No macs were found on the system')
             raise exception.IloError(msg)
 
-        iscsi_uri = self._check_iscsi_rest_patch_allowed()
-        # Set iSCSI info to all nics
         iscsi_infos = []
-        for nic in nics:
+        for association_name in association_names:
             data = iscsi_info.copy()
-            data['iSCSIBootAttemptName'] = nic
-            data['iSCSINicSource'] = nic
-            data['iSCSIBootAttemptInstance'] = nics.index(nic) + 1
+            data['iSCSIBootAttemptName'] = association_name
+            data['iSCSINicSource'] = association_name
+            data['iSCSIBootAttemptInstance'] = (
+                association_names.index(association_name) + 1)
             iscsi_infos.append(data)
 
-        patch_data = {'iSCSIBootSources': iscsi_infos}
+        iscsi_data = {'iSCSIBootSources': iscsi_infos}
+
         status, headers, response = self._rest_patch(iscsi_uri,
-                                                     None, patch_data)
+                                                     None, iscsi_data)
         if status >= 300:
             msg = self._get_extended_error(response)
             raise exception.IloError(msg)
@@ -924,7 +980,7 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
 
     def set_iscsi_info(self, target_name, lun, ip_address,
                        port='3260', auth_method=None, username=None,
-                       password=None):
+                       password=None, macs=[]):
         """Set iSCSI details of the system in UEFI boot mode.
 
         The initiator system is set with the target details like
@@ -936,6 +992,7 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
         :param auth_method : either None or CHAP.
         :param username: CHAP Username for authentication.
         :param password: CHAP secret.
+        :param macs: List of target macs for iSCSI.
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedInBiosError, if the system is
                  in the BIOS boot mode.
@@ -952,21 +1009,22 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                 iscsi_info['iSCSIAuthenticationMethod'] = 'Chap'
                 iscsi_info['iSCSIChapUsername'] = username
                 iscsi_info['iSCSIChapSecret'] = password
-            self._change_iscsi_settings(iscsi_info)
+            self._change_iscsi_settings(iscsi_info, macs)
         else:
             msg = 'iSCSI boot is not supported in the BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
 
-    def unset_iscsi_info(self):
+    def unset_iscsi_info(self, macs=[]):
         """Disable iSCSI boot option in UEFI boot mode.
 
+        :param macs: List of target macs for iSCSI.
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedInBiosError, if the system is
                  in the BIOS boot mode.
         """
         if(self._is_boot_mode_uefi() is True):
             iscsi_info = {'iSCSIBootEnable': 'Disabled'}
-            self._change_iscsi_settings(iscsi_info)
+            self._change_iscsi_settings(iscsi_info, macs)
         else:
             msg = 'iSCSI boot is not supported in the BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
@@ -1853,7 +1911,7 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
         raise exception.IloError("Attribute 'Oem/Hp/PostState' "
                                  "not found on system.")
 
-    def get_current_bios_settings(self, only_allowed_settings=True):
+    def get_current_bios_settings(self, only_allowed_settings=False):
         """Get current BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1875,7 +1933,7 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                 bios_settings, constants.SUPPORTED_BIOS_PROPERTIES)
         return bios_settings
 
-    def get_pending_bios_settings(self, only_allowed_settings=True):
+    def get_pending_bios_settings(self, only_allowed_settings=False):
         """Get current BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1910,7 +1968,7 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                 config, constants.SUPPORTED_BIOS_PROPERTIES)
         return config
 
-    def set_bios_settings(self, data=None, only_allowed_settings=True):
+    def set_bios_settings(self, data=None, only_allowed_settings=False):
         """Sets current BIOS settings to the provided data.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1924,16 +1982,24 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
         :raises: IloCommandNotSupportedError, if the command is not supported
                  on the server.
         """
-        if only_allowed_settings and data:
-            refined_data = utils.apply_bios_properties_filter(
-                data, constants.SUPPORTED_BIOS_PROPERTIES)
-            self._change_bios_setting(refined_data)
-            return
+        if not data:
+            raise exception.IloError("Could not apply settings with"
+                                     " empty data")
 
-        if data:
-            self._change_bios_setting(data)
+        if only_allowed_settings:
+            unsupported_settings = [key for key in data if key not in (
+                constants.SUPPORTED_BIOS_PROPERTIES)]
+            if unsupported_settings:
+                msg = ("Could not apply settings as one or more settings are"
+                       " not supported. Unsupported settings are %s."
+                       " Supported settings are %s." % (
+                           unsupported_settings,
+                           constants.SUPPORTED_BIOS_PROPERTIES))
+                raise exception.IloError(msg)
 
-    def get_default_bios_settings(self, only_allowed_settings=True):
+        self._change_bios_setting(data)
+
+    def get_default_bios_settings(self, only_allowed_settings=False):
         """Get default BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1981,13 +2047,57 @@ class RISOperations(rest.RestConnectorBase, operations.IloOperations):
                {'method': method, 'platform': platform})
         raise (exception.IloCommandNotSupportedError(msg))
 
+    def read_raid_configuration(self, raid_config=None):
+        """Read the logical drives from the system
+
+        Read raid configuration of the hardware.
+
+        :param raid_config: None in case of post-delete read or in case of
+                            post-create a dictionary containing target raid
+                            configuration data. This data stucture should be as
+                            follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloCommandNotSupportedError, if the command is not supported
+                 on the server.
+        """
+        self._raise_command_not_supported("read_raid_configuration")
+
     def delete_raid_configuration(self):
         """Delete the raid configuration on the hardware.
 
         Loops through each SmartStorageConfig controller and clears the
         raid configuration.
 
-        :raises: IloError, on an error from iLO
         :raises: IloCommandNotSupportedError
         """
         self._raise_command_not_supported("delete_raid_configuration")
+
+    def create_raid_configuration(self, raid_config):
+        """Create the raid configuration on the hardware.
+
+        Based on user raid_config input, it will create raid
+
+        :param raid_config: A dictionary containing target raid configuration
+                            data. This data stucture should be as follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloCommandNotSupportedError
+        """
+        self._raise_command_not_supported("create_raid_configuration")
+
+    def get_bios_settings_result(self):
+        """Gets the result of the bios settings applied
+
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the command is
+                 not supported on the server.
+        """
+        headers, bios_uri, bios_settings = self._check_bios_resource()
+        settings_result = bios_settings.get("SettingsResult").get("Messages")
+        status = "failed" if len(settings_result) > 1 else "success"
+        return {"status": status, "results": settings_result}
