@@ -18,7 +18,6 @@ import json
 
 from six.moves.urllib import parse
 import sushy
-from sushy import auth
 from sushy.resources.system import mappings as sushy_map
 from sushy import utils
 
@@ -153,10 +152,9 @@ class RedfishOperations(operations.IloOperations):
         self._username = username
 
         try:
-            basic_auth = auth.BasicAuth(username=username, password=password)
             self._sushy = main.HPESushy(
-                address, root_prefix=root_prefix, verify=verify,
-                auth=basic_auth)
+                address, username=username, password=password,
+                root_prefix=root_prefix, verify=verify)
         except sushy.exceptions.SushyError as e:
             msg = (self._('The Redfish controller at "%(controller)s" has '
                           'thrown error. Error %(error)s') %
@@ -164,6 +162,13 @@ class RedfishOperations(operations.IloOperations):
             LOG.debug(msg)
             raise exception.IloConnectionError(msg)
 
+    def __del__(self):
+        try:
+            if self._sushy:
+                self._sushy.close()
+        except AttributeError:
+            pass
+
     def _get_sushy_system(self, system_id):
         """Get the sushy system for system_id
 
@@ -393,8 +398,8 @@ class RedfishOperations(operations.IloOperations):
 
             LOG.debug(self._("Ejecting the media image '%(url)s' from the "
                              "device %(device)s.") %
-                      {'url': vmedia_device.image_url, 'device': device})
-            vmedia_device.eject_vmedia()
+                      {'url': vmedia_device.image, 'device': device})
+            vmedia_device.eject_media()
         except sushy.exceptions.SushyError as e:
             msg = (self._("The Redfish controller failed to eject the virtual"
                           " media device '%(device)s'. Error %(error)s.") %
@@ -417,12 +422,12 @@ class RedfishOperations(operations.IloOperations):
                 manager.virtual_media.get_member_device(
                     VIRTUAL_MEDIA_MAP[device]))
             if vmedia_device.inserted:
-                vmedia_device.eject_vmedia()
+                vmedia_device.eject_media()
 
             LOG.debug(self._("Inserting the image url '%(url)s' to the "
                              "device %(device)s.") %
                       {'url': url, 'device': device})
-            vmedia_device.insert_vmedia(url)
+            vmedia_device.insert_media(url)
         except sushy.exceptions.SushyError as e:
             msg = (self._("The Redfish controller failed to insert the media "
                           "url %(url)s in the virtual media device "
@@ -877,23 +882,47 @@ class RedfishOperations(operations.IloOperations):
             LOG.debug(msg)
             raise exception.IloError(msg)
 
-    def _change_iscsi_target_settings(self, iscsi_info):
+    def _change_iscsi_target_settings(self, iscsi_info, macs=[]):
         """Change iSCSI target settings.
 
+        :param macs: List of target mac for iSCSI.
         :param iscsi_info: A dictionary that contains information of iSCSI
                            target like target_name, lun, ip_address, port etc.
         :raises: IloError, on an error from iLO.
         """
         sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        association_names = []
         try:
-            pci_settings_map = (
-                sushy_system.bios_settings.bios_mappings.pci_settings_mappings)
-            nics = []
-            for mapping in pci_settings_map:
-                for subinstance in mapping['Subinstances']:
-                    for association in subinstance['Associations']:
-                        if 'NicBoot' in association:
-                            nics.append(association)
+            if macs:
+                sushy_system.validate_macs(macs)
+                association_names = [
+                    sushy_system.get_nic_association_name_by_mac(
+                        mac) for mac in macs]
+            else:
+                pci_settings_map = (
+                    sushy_system.bios_settings.
+                    bios_mappings.pci_settings_mappings)
+                for mapping in pci_settings_map:
+                    for subinstance in mapping['Subinstances']:
+                        for association in subinstance['Associations']:
+                            if 'NicBoot' in association:
+                                association_names.append(association)
+
+            if not association_names:
+                msg = ('No macs were found on the system')
+                raise exception.IloError(msg)
+
+            # Set iSCSI info to all nics
+            iscsi_infos = []
+            for association_name in association_names:
+                data = iscsi_info.copy()
+                data['iSCSIAttemptName'] = association_name
+                data['iSCSINicSource'] = association_name
+                data['iSCSIAttemptInstance'] = (
+                    association_names.index(association_name) + 1)
+                iscsi_infos.append(data)
+
+            iscsi_data = {'iSCSISources': iscsi_infos}
         except sushy.exceptions.SushyError as e:
             msg = (self._('The Redfish controller failed to get the '
                           'bios mappings. Error %(error)s')
@@ -901,20 +930,6 @@ class RedfishOperations(operations.IloOperations):
             LOG.debug(msg)
             raise exception.IloError(msg)
 
-        if not nics:
-            msg = ('No nics were found on the system')
-            raise exception.IloError(msg)
-
-        # Set iSCSI info to all nics
-        iscsi_infos = []
-        for nic in nics:
-            data = iscsi_info.copy()
-            data['iSCSIAttemptName'] = nic
-            data['iSCSINicSource'] = nic
-            data['iSCSIAttemptInstance'] = nics.index(nic) + 1
-            iscsi_infos.append(data)
-
-        iscsi_data = {'iSCSISources': iscsi_infos}
         try:
             (sushy_system.bios_settings.iscsi_resource.
              iscsi_settings.update_iscsi_settings(iscsi_data))
@@ -927,7 +942,7 @@ class RedfishOperations(operations.IloOperations):
 
     def set_iscsi_info(self, target_name, lun, ip_address,
                        port='3260', auth_method=None, username=None,
-                       password=None):
+                       password=None, macs=[]):
         """Set iSCSI details of the system in UEFI boot mode.
 
         The initiator system is set with the target details like
@@ -939,6 +954,7 @@ class RedfishOperations(operations.IloOperations):
         :param auth_method : either None or CHAP.
         :param username: CHAP Username for authentication.
         :param password: CHAP secret.
+        :param macs: List of target macs for iSCSI.
         :raises: IloCommandNotSupportedInBiosError, if the system is
                  in the bios boot mode.
         """
@@ -954,20 +970,21 @@ class RedfishOperations(operations.IloOperations):
                 iscsi_info['iSCSIAuthenticationMethod'] = 'Chap'
                 iscsi_info['iSCSIChapUsername'] = username
                 iscsi_info['iSCSIChapSecret'] = password
-            self._change_iscsi_target_settings(iscsi_info)
+            self._change_iscsi_target_settings(iscsi_info, macs)
         else:
             msg = 'iSCSI boot is not supported in the BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
 
-    def unset_iscsi_info(self):
+    def unset_iscsi_info(self, macs=[]):
         """Disable iSCSI boot option in UEFI boot mode.
 
+        :param macs: List of target macs for iSCSI.
         :raises: IloCommandNotSupportedInBiosError, if the system is
                  in the BIOS boot mode.
         """
         if(self._is_boot_mode_uefi()):
             iscsi_info = {'iSCSIConnection': 'Disabled'}
-            self._change_iscsi_target_settings(iscsi_info)
+            self._change_iscsi_target_settings(iscsi_info, macs)
         else:
             msg = 'iSCSI boot is not supported in the BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
@@ -1052,12 +1069,50 @@ class RedfishOperations(operations.IloOperations):
         sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
         return GET_POST_STATE_MAP.get(sushy_system.post_state)
 
+    def read_raid_configuration(self, raid_config=None):
+        """Read the logical drives from the system
+
+        :param raid_config: None in case of post-delete read or in case of
+                            post-create a dictionary containing target raid
+                            configuration data. This data stucture should be as
+                            follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloError, on an error from iLO.
+        :returns: A dictionary containing list of logical disks
+        """
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        return sushy_system.read_raid(raid_config=raid_config)
+
     def delete_raid_configuration(self):
         """Delete the raid configuration on the hardware."""
         sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
         sushy_system.delete_raid()
 
-    def get_current_bios_settings(self, only_allowed_settings=True):
+    def do_disk_erase(self, disk_type, pattern=None):
+        """Perform the out-of-band sanitize disk erase on the hardware.
+
+        :param disk_type: Media type of disk drives either 'HDD' or 'SSD'.
+        :param pattern: Erase pattern, if nothing passed default
+                        ('overwrite' for 'HDD', and 'block' for 'SSD') will
+                        be used.
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        sushy_system.do_disk_erase(disk_type, pattern)
+
+    def has_disk_erase_completed(self):
+        """Get out-of-band sanitize disk erase status.
+
+        :returns: True if disk erase completed on all controllers
+                  otherwise False
+        """
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        return sushy_system.has_disk_erase_completed()
+
+    def get_current_bios_settings(self, only_allowed_settings=False):
         """Get current BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1085,7 +1140,7 @@ class RedfishOperations(operations.IloOperations):
                 attributes, ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES)
         return attributes
 
-    def get_pending_bios_settings(self, only_allowed_settings=True):
+    def get_pending_bios_settings(self, only_allowed_settings=False):
         """Get pending BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings are
@@ -1113,7 +1168,7 @@ class RedfishOperations(operations.IloOperations):
                 attributes, ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES)
         return attributes
 
-    def set_bios_settings(self, data=None, only_allowed_settings=True):
+    def set_bios_settings(self, data=None, only_allowed_settings=False):
         """Sets current BIOS settings to the provided data.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1127,23 +1182,32 @@ class RedfishOperations(operations.IloOperations):
         :raises: IloCommandNotSupportedError, if the command is not supported
                  on the server.
         """
-        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
-        filtered_data = data
-        if only_allowed_settings and data:
-            filtered_data = common_utils.apply_bios_properties_filter(
-                data, ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES)
-        if filtered_data:
-            try:
-                settings_required = sushy_system.bios_settings.pending_settings
-                settings_required.update_bios_data_by_patch(filtered_data)
-            except sushy.exceptions.SushyError as e:
-                msg = (self._('The pending BIOS Settings resource not found.'
-                              ' Error %(error)s') %
-                       {'error': str(e)})
-                LOG.debug(msg)
-                raise exception.IloError(msg)
+        if not data:
+            raise exception.IloError("Could not apply settings with"
+                                     " empty data")
 
-    def get_default_bios_settings(self, only_allowed_settings=True):
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        if only_allowed_settings:
+            unsupported_settings = [key for key in data if key not in (
+                ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES)]
+            if unsupported_settings:
+                msg = ("Could not apply settings as one or more settings are"
+                       " not supported. Unsupported settings are %s."
+                       " Supported settings are %s." % (
+                           unsupported_settings,
+                           ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES))
+                raise exception.IloError(msg)
+        try:
+            settings_required = sushy_system.bios_settings.pending_settings
+            settings_required.update_bios_data_by_patch(data)
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The pending BIOS Settings resource not found.'
+                          ' Error %(error)s') %
+                   {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+
+    def get_default_bios_settings(self, only_allowed_settings=False):
         """Get default BIOS settings.
 
         :param: only_allowed_settings: True when only allowed BIOS settings
@@ -1168,3 +1232,47 @@ class RedfishOperations(operations.IloOperations):
             return common_utils.apply_bios_properties_filter(
                 settings, ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES)
         return settings
+
+    def create_raid_configuration(self, raid_config):
+        """Create the raid configuration on the hardware.
+
+        Based on user raid_config input, it will create raid
+
+        :param raid_config: A dictionary containing target raid configuration
+                            data. This data stucture should be as follows:
+                            raid_config = {'logical_disks': [{'raid_level': 1,
+                            'size_gb': 100, 'physical_disks': ['6I:1:5'],
+                            'controller': 'HPE Smart Array P408i-a SR Gen10'},
+                            <info-for-logical-disk-2>]}
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        sushy_system.create_raid(raid_config)
+
+    def get_bios_settings_result(self):
+        """Gets the result of the bios settings applied
+
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the command is
+                 not supported on the server.
+        """
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        try:
+            settings_result = sushy_system.bios_settings.messages
+        except sushy.exceptions.SushyError as e:
+            msg = (self._('The BIOS Settings results were not found. Error '
+                          '%(error)s') %
+                   {'error': str(e)})
+            LOG.debug(msg)
+            raise exception.IloError(msg)
+        status = "failed" if len(settings_result) > 1 else "success"
+        return {"status": status, "results": settings_result}
+
+    def get_available_disk_types(self):
+        """Get the list of all disk type available in server
+
+        :returns: A list containing disk types
+        :raises: IloError, on an error from iLO.
+        """
+        sushy_system = self._get_sushy_system(PROLIANT_SYSTEM_ID)
+        return sushy_system.get_disk_types()
