@@ -13,6 +13,7 @@
 #    under the License.
 
 from sushy.resources import base
+from sushy import utils as sushy_utils
 
 from proliantutils.redfish import utils
 
@@ -30,29 +31,16 @@ class Volume(base.ResourceBase):
 class VolumeCollection(base.ResourceCollectionBase):
     """This class represents the collection of Volume resource"""
 
-    _maximum_size_bytes = None
-
     @property
     def _resource_type(self):
         return Volume
 
     @property
+    @sushy_utils.cache_it
     def maximum_size_bytes(self):
         """Gets the biggest volume
 
         :returns size in bytes.
         """
-        if self._maximum_size_bytes is None:
-            self._maximum_size_bytes = (
-                utils.max_safe([member.capacity_bytes
-                               for member in self.get_members()]))
-        return self._maximum_size_bytes
-
-    def _do_refresh(self, force):
-        """Do custom resource specific refresh activities
-
-        On refresh, all sub-resources are marked as stale, i.e.
-        greedy-refresh not done for them unless forced by ``force``
-        argument.
-        """
-        self._maximum_size_bytes = None
+        return utils.max_safe([member.capacity_bytes
+                               for member in self.get_members()])
