@@ -1,4 +1,4 @@
-# Copyright 2017 Hewlett Packard Enterprise Development LP
+# Copyright 2019 Hewlett Packard Enterprise Development LP
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,7 +21,6 @@ import mock
 import sushy
 import testtools
 
-from sushy import auth
 from sushy.resources.system import system
 
 from proliantutils import exception
@@ -41,7 +40,6 @@ from proliantutils.redfish.resources.system.storage import array_controller
 from proliantutils.redfish.resources.system.storage \
     import common as common_storage
 from proliantutils.redfish.resources.system import system as pro_sys
-from proliantutils import utils
 
 
 @ddt.ddt
@@ -66,7 +64,8 @@ class RedfishOperationsTestCase(testtools.TestCase):
         self.assertEqual(('https://1.2.3.4',), args)
         self.assertFalse(kwargs.get('verify'))
         self.assertEqual('/redfish/v1/', kwargs.get('root_prefix'))
-        self.assertIsInstance(kwargs.get('auth'), auth.BasicAuth)
+        self.assertEqual('foo', kwargs.get('username'))
+        self.assertEqual('bar', kwargs.get('password'))
 
     @mock.patch.object(main, 'HPESushy', autospec=True)
     def test_sushy_init_fail(self, sushy_mock):
@@ -265,7 +264,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
         return manager_mock, vmedia_collection_json, vmedia_json
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
     def test_eject_virtual_media(self, eject_mock, manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
             self._setup_virtual_media())
@@ -277,7 +276,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
         eject_mock.assert_called_once_with()
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
     def test_eject_virtual_media_invalid_device(self, eject_mock,
                                                 manager_mock):
         self.assertRaisesRegex(
@@ -290,7 +289,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
         self.assertFalse(manager_mock.called)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
     def test_eject_virtual_media_not_inserted(self, eject_mock, manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
             self._setup_virtual_media())
@@ -302,7 +301,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
         self.assertFalse(eject_mock.called)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
     def test_eject_virtual_media_floppy(self, eject_mock, manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
             self._setup_virtual_media())
@@ -314,7 +313,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
         self.assertFalse(eject_mock.called)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
     def test_eject_virtual_media_fail(self, eject_mock, manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
             self._setup_virtual_media())
@@ -329,8 +328,8 @@ class RedfishOperationsTestCase(testtools.TestCase):
                                'CDROM')
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
-    @mock.patch.object(virtual_media.VirtualMedia, 'insert_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
+    @mock.patch.object(virtual_media.VirtualMedia, 'insert_media')
     def test_insert_virtual_media(self, insert_mock, eject_mock, manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
             self._setup_virtual_media())
@@ -344,8 +343,8 @@ class RedfishOperationsTestCase(testtools.TestCase):
         insert_mock.assert_called_once_with(url)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
-    @mock.patch.object(virtual_media.VirtualMedia, 'insert_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
+    @mock.patch.object(virtual_media.VirtualMedia, 'insert_media')
     def test_insert_virtual_media_floppy(self, insert_mock, eject_mock,
                                          manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
@@ -360,8 +359,8 @@ class RedfishOperationsTestCase(testtools.TestCase):
         insert_mock.assert_called_once_with(url)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
-    @mock.patch.object(virtual_media.VirtualMedia, 'insert_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
+    @mock.patch.object(virtual_media.VirtualMedia, 'insert_media')
     def test_insert_virtual_media_inserted(self, insert_mock, eject_mock,
                                            manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
@@ -376,8 +375,8 @@ class RedfishOperationsTestCase(testtools.TestCase):
         insert_mock.assert_called_once_with(url)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_manager')
-    @mock.patch.object(virtual_media.VirtualMedia, 'eject_vmedia')
-    @mock.patch.object(virtual_media.VirtualMedia, 'insert_vmedia')
+    @mock.patch.object(virtual_media.VirtualMedia, 'eject_media')
+    @mock.patch.object(virtual_media.VirtualMedia, 'insert_media')
     def test_insert_virtual_media_fail(self, insert_mock, eject_mock,
                                        manager_mock):
         manager_mock.return_value, vmedia_collection_json, vmedia_json = (
@@ -1094,7 +1093,27 @@ class RedfishOperationsTestCase(testtools.TestCase):
             'iqn.2011-07.com.example.server:test1',
             '1', '10.10.1.30')
         change_iscsi_target_settings_mock.assert_called_once_with(
-            self.rf_client, iscsi_variables)
+            self.rf_client, iscsi_variables, [])
+
+    @mock.patch.object(redfish.RedfishOperations,
+                       '_change_iscsi_target_settings', autospec=True)
+    @mock.patch.object(redfish.RedfishOperations, '_is_boot_mode_uefi',
+                       autospec=True)
+    def test_set_iscsi_info_uefi_with_mac(self, _uefi_boot_mode_mock,
+                                          change_iscsi_target_settings_mock):
+        _uefi_boot_mode_mock.return_value = True
+        iscsi_variables = {
+            'iSCSITargetName': 'iqn.2011-07.com.example.server:test1',
+            'iSCSITargetInfoViaDHCP': False,
+            'iSCSILUN': '1',
+            'iSCSIConnection': 'Enabled',
+            'iSCSITargetIpAddress': '10.10.1.30',
+            'iSCSITargetTcpPort': 3260}
+        self.rf_client.set_iscsi_info(
+            'iqn.2011-07.com.example.server:test1',
+            '1', '10.10.1.30', macs=['98:f2:b3:ee:f4:00'])
+        change_iscsi_target_settings_mock.assert_called_once_with(
+            self.rf_client, iscsi_variables, ['98:f2:b3:ee:f4:00'])
 
     @mock.patch.object(redfish.RedfishOperations,
                        '_change_iscsi_target_settings', autospec=True)
@@ -1117,7 +1136,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
             'iqn.2011-07.com.example.server:test1',
             '1', '10.10.1.30', 3260, 'CHAP', 'admin', 'password')
         change_iscsi_target_settings_mock.assert_called_once_with(
-            self.rf_client, iscsi_variables)
+            self.rf_client, iscsi_variables, [])
 
     @mock.patch.object(redfish.RedfishOperations, '_is_boot_mode_uefi',
                        autospec=True)
@@ -1139,7 +1158,20 @@ class RedfishOperationsTestCase(testtools.TestCase):
             'iSCSIConnection': 'Disabled'}
         self.rf_client.unset_iscsi_info()
         change_iscsi_target_settings_mock.assert_called_once_with(
-            self.rf_client, iscsi_variables)
+            self.rf_client, iscsi_variables, [])
+
+    @mock.patch.object(redfish.RedfishOperations,
+                       '_change_iscsi_target_settings', autospec=True)
+    @mock.patch.object(redfish.RedfishOperations, '_is_boot_mode_uefi',
+                       autospec=True)
+    def test_unset_iscsi_info_uefi_with_mac(self, _uefi_boot_mode_mock,
+                                            change_iscsi_target_settings_mock):
+        _uefi_boot_mode_mock.return_value = True
+        iscsi_variables = {
+            'iSCSIConnection': 'Disabled'}
+        self.rf_client.unset_iscsi_info(['98:f2:b3:ee:f4:00'])
+        change_iscsi_target_settings_mock.assert_called_once_with(
+            self.rf_client, iscsi_variables, ['98:f2:b3:ee:f4:00'])
 
     @mock.patch.object(iscsi.ISCSISettings, 'update_iscsi_settings')
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
@@ -1201,10 +1233,66 @@ class RedfishOperationsTestCase(testtools.TestCase):
         data = {
             'iSCSISources': [iscsi_data1, iscsi_data2]
         }
-        self.rf_client._change_iscsi_target_settings(iscsi_variable)
+        self.rf_client._change_iscsi_target_settings(iscsi_variable, [])
         update_iscsi_settings_mock.assert_called_once_with(
             data)
 
+    @mock.patch.object(pro_sys.HPESystem, 'validate_macs')
+    @mock.patch.object(pro_sys.HPESystem, 'get_nic_association_name_by_mac')
+    @mock.patch.object(iscsi.ISCSISettings, 'update_iscsi_settings')
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test__change_iscsi_target_settings_with_mac(
+            self, get_system_mock, update_iscsi_settings_mock,
+            get_nic_association_mock, validate_macs_mock):
+        with open('proliantutils/tests/redfish/'
+                  'json_samples/system.json', 'r') as f:
+            system_json = json.loads(f.read())
+        with open('proliantutils/tests/redfish/'
+                  'json_samples/bios.json', 'r') as f:
+            bios_json = json.loads(f.read())
+        with open('proliantutils/tests/redfish/'
+                  'json_samples/iscsi.json', 'r') as f:
+            iscsi_json = json.loads(f.read())
+        with open('proliantutils/tests/redfish/'
+                  'json_samples/iscsi_settings.json', 'r') as f:
+            iscsi_settings_json = json.loads(f.read())
+
+        self.conn = mock.Mock()
+        self.conn.get.return_value.json.side_effect = [
+            system_json['default'], bios_json['Default'],
+            iscsi_json,
+            iscsi_settings_json['Default']]
+        self.sys_inst = pro_sys.HPESystem(self.conn,
+                                          '/redfish/v1/Systems/437XR1138R2',
+                                          redfish_version='1.0.2')
+        get_system_mock.return_value = self.sys_inst
+        get_nic_association_mock.return_value = 'NicBoot1'
+        iscsi_variable = {'iSCSITargetName':
+                          'iqn.2011-07.com.example.server:test1',
+                          'iSCSILUN': '1',
+                          'iSCSITargetIpAddress': '10.10.1.30',
+                          'iSCSITargetTcpPort': 3260,
+                          'iSCSITargetInfoViaDHCP': False,
+                          'iSCSIConnection': 'Enabled'}
+        iscsi_data = {'iSCSITargetName':
+                      'iqn.2011-07.com.example.server:test1',
+                      'iSCSILUN': '1',
+                      'iSCSITargetIpAddress': '10.10.1.30',
+                      'iSCSITargetTcpPort': 3260,
+                      'iSCSITargetInfoViaDHCP': False,
+                      'iSCSIConnection': 'Enabled',
+                      'iSCSIAttemptName': 'NicBoot1',
+                      'iSCSINicSource': 'NicBoot1',
+                      'iSCSIAttemptInstance': 1}
+        data = {
+            'iSCSISources': [iscsi_data]
+        }
+        self.rf_client._change_iscsi_target_settings(
+            iscsi_variable, ['98:f2:b3:ee:f4:00'])
+        update_iscsi_settings_mock.assert_called_once_with(
+            data)
+        validate_macs_mock.assert_called_once_with(['98:f2:b3:ee:f4:00'])
+
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
     def test__change_iscsi_target_settings_failed_getting_mappings(
             self, get_system_mock):
@@ -1216,11 +1304,10 @@ class RedfishOperationsTestCase(testtools.TestCase):
             exception.IloError,
             "The Redfish controller failed to get the "
             "bios mappings. Error",
-            self.rf_client._change_iscsi_target_settings,
-            '{}')
+            self.rf_client._change_iscsi_target_settings, {}, [])
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
-    def test__change_iscsi_target_settings_no_nics(
+    def test__change_iscsi_target_settings_no_macs(
             self, get_system_mock):
         with open('proliantutils/tests/redfish/'
                   'json_samples/system.json', 'r') as f:
@@ -1241,9 +1328,21 @@ class RedfishOperationsTestCase(testtools.TestCase):
         get_system_mock.return_value = self.sys_inst
         self.assertRaisesRegex(
             exception.IloError,
-            "No nics were found on the system",
-            self.rf_client._change_iscsi_target_settings,
-            '{}')
+            "No macs were found on the system",
+            self.rf_client._change_iscsi_target_settings, {}, [])
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test__change_iscsi_target_settings_mac_invalid(
+            self, get_system_mock):
+        msg = ("Given macs: %(macs)s not found in the system"
+               % {'macs': str(['12:44:6A:3B:04:15'])})
+        get_system_mock.return_value.validate_macs.side_effect = (
+            exception.InvalidInputError(msg))
+        self.assertRaisesRegex(
+            exception.InvalidInputError,
+            "Given macs: \['12:44:6A:3B:04:15'\] not found in the system",
+            self.rf_client._change_iscsi_target_settings, {},
+            ['12:44:6A:3B:04:15'])
 
     @mock.patch.object(iscsi.ISCSISettings, 'update_iscsi_settings')
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
@@ -1288,8 +1387,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
             exception.IloError,
             'The Redfish controller is failed to update iSCSI '
             'settings.',
-            self.rf_client._change_iscsi_target_settings,
-            iscsi_variable)
+            self.rf_client._change_iscsi_target_settings, iscsi_variable, [])
 
     @mock.patch.object(iscsi.ISCSISettings, 'update_iscsi_settings')
     @mock.patch.object(redfish.RedfishOperations, '_is_boot_mode_uefi',
@@ -1584,49 +1682,76 @@ class RedfishOperationsTestCase(testtools.TestCase):
             only_allowed_settings)
 
     @mock.patch.object(bios.BIOSPendingSettings, 'update_bios_data_by_patch')
-    @mock.patch.object(utils, 'apply_bios_properties_filter')
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
-    def test_set_bios_settings_no_data(self, system_mock, bios_filter_mock,
-                                       update_data_mock):
+    def test_set_bios_settings_no_data(self, system_mock, update_data_mock):
         data = None
         apply_filter = True
-        pending_settings_mock = mock.PropertyMock()
-        type(system_mock.return_value.bios_settings).pending_settings = (
-            pending_settings_mock)
-        self.rf_client.set_bios_settings(data, apply_filter)
-        bios_filter_mock.assert_not_called()
-        pending_settings_mock.assert_not_called()
+        self.assertRaisesRegex(
+            exception.IloError,
+            "Could not apply settings with empty data",
+            self.rf_client.set_bios_settings,
+            data, apply_filter)
+        update_data_mock.assert_not_called()
+        system_mock.assert_not_called()
+
+    @mock.patch.object(bios.BIOSPendingSettings, 'update_bios_data_by_patch')
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_set_bios_settings_no_data_no_filter(self, system_mock,
+                                                 update_data_mock):
+
+        data = None
+        apply_filter = False
+        self.assertRaisesRegex(
+            exception.IloError,
+            "Could not apply settings with empty data",
+            self.rf_client.set_bios_settings,
+            data, apply_filter)
+        update_data_mock.assert_not_called()
+        system_mock.assert_not_called()
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
-    def test_set_bios_settings_filter_true(self, system_mock):
+    def test_set_bios_settings_filter_true_valid_data(self, system_mock):
         apply_filter = True
         data = {
-            "AdminName": "Administrator",
-            "BootMode": "LEGACY",
-            "ServerName": "Gen9 server",
-            "TimeFormat": "Ist",
-            "BootOrderPolicy": "RetryIndefinitely",
-            "ChannelInterleaving": "Enabled",
-            "CollabPowerControl": "Enabled",
-            "ConsistentDevNaming": "LomsOnly",
-            "CustomPostMessage": ""
+            "BootOrderPolicy": "AttemptOnce",
+            "IntelPerfMonitoring": "Enabled",
+            "IntelProcVtd": "Disabled",
+            "UefiOptimizedBoot": "Disabled",
+            "PowerProfile": "MaxPerf",
         }
-        expected = {k: data[k] for k in data if k in (
-            ilo_cons.SUPPORTED_REDFISH_BIOS_PROPERTIES)}
         bios_ps_mock = mock.MagicMock(spec=bios.BIOSPendingSettings)
         pending_settings_mock = mock.PropertyMock(return_value=bios_ps_mock)
         type(system_mock.return_value.bios_settings).pending_settings = (
             pending_settings_mock)
 
         self.rf_client.set_bios_settings(data, apply_filter)
-        bios_ps_mock.update_bios_data_by_patch.assert_called_once_with(
-            expected)
+        bios_ps_mock.update_bios_data_by_patch.assert_called_once_with(data)
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_set_bios_settings_filter_true_invalid_data(self, system_mock):
+        apply_filter = True
+        data = {
+            "AdminName": "Administrator",
+            "BootOrderPolicy": "AttemptOnce",
+            "IntelPerfMonitoring": "Enabled",
+            "IntelProcVtd": "Disabled",
+            "UefiOptimizedBoot": "Disabled",
+            "PowerProfile": "MaxPerf",
+            "TimeZone": "Utc1"
+        }
+
+        self.assertRaisesRegex(
+            exception.IloError,
+            "Could not apply settings as one or more settings"
+            " are not supported",
+            self.rf_client.set_bios_settings,
+            data, apply_filter)
+        system_mock.assert_called_once_with(redfish.PROLIANT_SYSTEM_ID)
 
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
     def test_set_bios_settings_filter_false(self, system_mock):
         apply_filter = False
         data = {
-            "AdminName": "Administrator",
             "BootMode": "LEGACY",
             "ServerName": "Gen9 server",
             "TimeFormat": "Ist",
@@ -1636,6 +1761,7 @@ class RedfishOperationsTestCase(testtools.TestCase):
             "ConsistentDevNaming": "LomsOnly",
             "CustomPostMessage": ""
         }
+
         bios_ps_mock = mock.MagicMock(spec=bios.BIOSPendingSettings)
         pending_settings_mock = mock.PropertyMock(return_value=bios_ps_mock)
         type(system_mock.return_value.bios_settings).pending_settings = (
@@ -1648,11 +1774,13 @@ class RedfishOperationsTestCase(testtools.TestCase):
     def test_set_bios_settings_raises_exception(self, system_mock):
         apply_filter = True
         data = {
-            "BootMode": "LEGACY",
-            "TimeFormat": "Ist",
-            "BootOrderPolicy": "RetryIndefinitely",
-            "CollabPowerControl": "Enabled",
+            "BootOrderPolicy": "AttemptOnce",
+            "IntelPerfMonitoring": "Enabled",
+            "IntelProcVtd": "Disabled",
+            "UefiOptimizedBoot": "Disabled",
+            "PowerProfile": "MaxPerf"
         }
+
         pending_settings_mock = mock.PropertyMock(
             side_effect=sushy.exceptions.SushyError)
         type(system_mock.return_value.bios_settings).pending_settings = (
@@ -1671,7 +1799,118 @@ class RedfishOperationsTestCase(testtools.TestCase):
         result = self.rf_client.get_host_post_state()
         self.assertEqual('PowerOff', result)
 
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_do_disk_erase_hdd(self, get_system_mock):
+        self.rf_client.do_disk_erase('HDD')
+        get_system_mock.return_value.do_disk_erase.assert_called_once_with(
+            'HDD', None)
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_do_disk_erase_ssd(self, get_system_mock):
+        self.rf_client.do_disk_erase('SSD')
+        get_system_mock.return_value.do_disk_erase.assert_called_once_with(
+            'SSD', None)
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_do_disk_erase_ssd_pattern_zero(self, get_system_mock):
+        self.rf_client.do_disk_erase('SSD', 'zero')
+        get_system_mock.return_value.do_disk_erase.assert_called_once_with(
+            'SSD', 'zero')
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_has_disk_erase_completed(self, get_system_mock):
+        (get_system_mock.return_value.
+         has_disk_erase_completed.return_value) = True
+        self.assertEqual(True, self.rf_client.has_disk_erase_completed())
+
     @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
     def test_delete_raid_configuration(self, get_system_mock):
         self.rf_client.delete_raid_configuration()
         get_system_mock.return_value.delete_raid.assert_called_once_with()
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_create_raid_configuration(self, get_system_mock):
+        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
+        raid_config = {"logical_disks": [ld1]}
+        self.rf_client.create_raid_configuration(raid_config)
+        get_system_mock.return_value.create_raid.assert_called_once_with(
+            raid_config)
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_read_raid_configuration(self, get_system_mock):
+        result_ld1 = [{'size_gb': 149,
+                       'physical_disks': [u'2I:1:1'],
+                       'raid_level': u'0',
+                       'root_device_hint': {'wwn': u'0x600508B'},
+                       'controller': u'Smart Storage Controller in Slot 1',
+                       'volume_name': u'01E6E63APFJHD'}]
+        config = {'logical_disks': result_ld1}
+        expected = [('HPE Smart Array P408i-p SR Gen10', config)]
+        get_system_mock.return_value.read_raid.return_value = expected
+        self.assertEqual(expected, self.rf_client.read_raid_configuration())
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_get_bios_settings_result_failed(self, get_system_mock):
+        with open('proliantutils/tests/redfish/'
+                  'json_samples/bios_failed.json', 'r') as f:
+            jsonval = json.loads(f.read()).get("Default")
+
+        type(get_system_mock.return_value.bios_settings).messages = (
+            jsonval['@Redfish.Settings']['Messages'])
+
+        expected_settings = [
+            {
+                "MessageArgs": [
+                    "MinProcIdlePkgState"
+                ],
+                "MessageID": "Base.1.0:PropertyNotWritable"
+            },
+            {
+                "MessageArgs": [
+                    "MinProcIdlePower"
+                ],
+                "MessageID": "Base.1.0:PropertyNotWritable"
+            },
+            {
+                "MessageArgs": [
+                    "EnergyPerfBias"
+                ],
+                "MessageID": "Base.1.0:PropertyNotWritable"
+            },
+            {
+                "MessageArgs": [
+                    "PowerRegulator"
+                ],
+                "MessageID": "Base.1.0:PropertyNotWritable"
+            },
+            {
+                "MessageArgs": [],
+                "MessageID": "Base.1.0:Success"
+            }
+        ]
+        expected = {"status": "failed", "results": expected_settings}
+        actual = self.rf_client.get_bios_settings_result()
+        self.assertEqual(expected, actual)
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_get_bios_settings_result_success(self, get_system_mock):
+        with open('proliantutils/tests/redfish/'
+                  'json_samples/bios.json', 'r') as f:
+            jsonval = json.loads(f.read()).get("Default")
+        actual_settings = [
+            {
+                "MessageId": "Base.1.0.Success"
+            }
+        ]
+        type(get_system_mock.return_value.bios_settings).messages = (
+            jsonval['@Redfish.Settings']['Messages'])
+        actual = self.rf_client.get_bios_settings_result()
+        expected = {"status": "success", "results": actual_settings}
+        self.assertEqual(expected, actual)
+
+    @mock.patch.object(redfish.RedfishOperations, '_get_sushy_system')
+    def test_get_available_disk_types(self, get_system_mock):
+        get_system_mock.return_value.get_disk_types.return_value = ['HDD',
+                                                                    'SSD']
+        self.assertEqual(
+            ['HDD', 'SSD'], self.rf_client.get_available_disk_types())
